# supported species names
species = ['Homo_sapiens','Mus_musculus']
# converting Names to database locations
databases = {
    'Human':'db/Homo_sapiens.db',
    'Homo_sapiens':'db/Homo_sapiens.db',
    'Mouse': 'db/Mus_musculus.db',
    'Mus_musculus': 'db/Mus_musculus.db'
}
